@@ -1,7 +1,5 @@
 """Provide types of the intermediate representation."""
 import abc
-import ast
-import collections
 import enum
 import pathlib
 from typing import (
@@ -11,12 +9,11 @@ from typing import (
     TypeVar,
     Mapping,
     MutableMapping,
-    List,
-    Tuple,
     Final,
     FrozenSet,
-    OrderedDict,
     Set,
+    Tuple,
+    OrderedDict,
 )
 
 import docutils.nodes
@@ -26,8 +23,8 @@ from aas_core_codegen import parse
 from aas_core_codegen.common import (
     Identifier,
     assert_never,
-    Error,
     assert_union_of_descendants_exhaustive,
+    assert_union_without_excluded,
 )
 from aas_core_codegen.intermediate import construction
 from aas_core_codegen.parse import tree as parse_tree
@@ -127,31 +124,40 @@ class OptionalTypeAnnotation(TypeAnnotation):
         return f"Optional[{self.value}]"
 
 
-class RefTypeAnnotation(TypeAnnotation):
-    """Represent a type annotation involving a reference ``Ref[...]``."""
-
-    def __init__(self, value: "TypeAnnotationUnion", parsed: parse.TypeAnnotation):
-        TypeAnnotation.__init__(self, parsed=parsed)
-
-        self.value = value
-
-    def __str__(self) -> str:
-        return f"Ref[{self.value}]"
-
-
 TypeAnnotationUnion = Union[
     PrimitiveTypeAnnotation,
     OurTypeAnnotation,
     ListTypeAnnotation,
     OptionalTypeAnnotation,
-    RefTypeAnnotation,
 ]
 
+
 assert_union_of_descendants_exhaustive(
     union=TypeAnnotationUnion, base_class=TypeAnnotation
 )
 
 
+TypeAnnotationExceptOptional = Union[
+    PrimitiveTypeAnnotation,
+    OurTypeAnnotation,
+    ListTypeAnnotation,
+]
+
+assert_union_without_excluded(
+    original_union=TypeAnnotationUnion,
+    subset_union=TypeAnnotationExceptOptional,
+    excluded=[OptionalTypeAnnotation],
+)
+
+AtomicTypeAnnotation = Union[PrimitiveTypeAnnotation, OurTypeAnnotation]
+
+assert_union_without_excluded(
+    original_union=TypeAnnotationUnion,
+    subset_union=AtomicTypeAnnotation,
+    excluded=[ListTypeAnnotation, OptionalTypeAnnotation],
+)
+
+
 def type_annotations_equal(
     that: TypeAnnotationUnion, other: TypeAnnotationUnion
 ) -> bool:
@@ -179,24 +185,192 @@ def type_annotations_equal(
         assert isinstance(other, OptionalTypeAnnotation)
         return type_annotations_equal(that.value, other.value)
 
-    elif isinstance(that, RefTypeAnnotation):
-        assert isinstance(other, RefTypeAnnotation)
-        return type_annotations_equal(that.value, other.value)
-
     else:
         assert_never(that)
 
     raise AssertionError("Should not have gotten here")
 
 
-class Description:
-    """Represent a docstring describing something in the meta-model."""
+def beneath_optional(
+    type_annotation: TypeAnnotationUnion,
+) -> TypeAnnotationExceptOptional:
+    """Descend below ``Optional[...]`` to the underlying type."""
+    type_anno = type_annotation
+    while isinstance(type_anno, OptionalTypeAnnotation):
+        type_anno = type_anno.value
 
-    @require(lambda node: isinstance(node.value, str))
-    def __init__(self, document: docutils.nodes.document, node: ast.Constant) -> None:
+    assert not isinstance(type_anno, OptionalTypeAnnotation)
+
+    return type_anno
+
+
+# region Descriptions
+
+# NOTE (mristin, 2022-03-18):
+# We take C# documentation comments as an orientation for the structure of the
+# descriptions.
+
+
+def find_first_field_list(
+    element: docutils.nodes.Element,
+) -> Optional[docutils.nodes.field_list]:
+    """Find the first field list beneath the element or return None."""
+    return next(element.findall(condition=docutils.nodes.field_list), None)
+
+
+class SummaryRemarksDescription(DBC):
+    """Represent a description with a summary and remarks."""
+
+    #: Summary as the first line of the docstring
+    summary: Final[docutils.nodes.paragraph]
+
+    #: List of remarks following the summary in the docstring
+    remarks: Final[Sequence[docutils.nodes.Element]]
+
+    #: Original parsed description
+    parsed: Final[parse.Description]
+
+    # fmt: off
+    @require(
+        lambda summary:
+        find_first_field_list(summary) is None,
+        "Summary expected without field lists"
+    )
+    @require(
+        lambda remarks:
+        all(
+            find_first_field_list(remark) is None
+            for remark in remarks
+        ),
+        "Remarks expected without field lists"
+    )
+    # fmt: on
+    def __init__(
+        self,
+        summary: docutils.nodes.paragraph,
+        remarks: Sequence[docutils.nodes.Element],
+        parsed: parse.Description,
+    ) -> None:
         """Initialize with the given values."""
-        self.document = document
-        self.node = node
+        self.summary = summary
+        self.remarks = remarks
+        self.parsed = parsed
+
+    @abc.abstractmethod
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        raise NotImplementedError()
+
+
+# noinspection PyAbstractClass
+class SummaryRemarksConstraintsDescription(SummaryRemarksDescription):
+    """Represent a description with summary, remarks and constraints blocks."""
+
+    #: Map constraint documentation elements by their identifiers
+    constraints_by_identifier: Final[OrderedDict[str, docutils.nodes.field_body]]
+
+    # fmt: off
+    @require(
+        lambda constraints_by_identifier:
+        all(
+            find_first_field_list(body) is None
+            for body in constraints_by_identifier.values()
+        ),
+        "Constraint bodies expected without field lists"
+    )
+    # fmt: on
+    def __init__(
+        self,
+        summary: docutils.nodes.paragraph,
+        remarks: Sequence[docutils.nodes.Element],
+        constraints_by_identifier: OrderedDict[str, docutils.nodes.field_body],
+        parsed: parse.Description,
+    ) -> None:
+        """Initialize with the given values."""
+        SummaryRemarksDescription.__init__(
+            self, summary=summary, remarks=remarks, parsed=parsed
+        )
+        self.constraints_by_identifier = constraints_by_identifier
+
+
+class MetaModelDescription(SummaryRemarksConstraintsDescription):
+    """Represent a description of a meta-model."""
+
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return f"<{_MODULE_NAME}.{self.__class__.__name__} at 0x{id(self):x}>"
+
+
+class SymbolDescription(SummaryRemarksConstraintsDescription):
+    """Represent a description of a symbol."""
+
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return f"<{_MODULE_NAME}.{self.__class__.__name__} at 0x{id(self):x}>"
+
+
+class PropertyDescription(SummaryRemarksConstraintsDescription):
+    """Represent a documentation of a property."""
+
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return f"<{_MODULE_NAME}.{self.__class__.__name__} at 0x{id(self):x}>"
+
+
+class EnumerationLiteralDescription(SummaryRemarksDescription):
+    """Represent a documentation of an enumeration literal."""
+
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return f"<{_MODULE_NAME}.{self.__class__.__name__} at 0x{id(self):x}>"
+
+
+class SignatureDescription(SummaryRemarksDescription):
+    """Represent a documentation of a method or a function signature."""
+
+    #: Map argument documentation by the argument names
+    arguments_by_name: Final[OrderedDict[Identifier, docutils.nodes.field_body]]
+
+    #: Documentation of the return value, if written
+    returns: Final[Optional[docutils.nodes.field_body]]
+
+    # fmt: off
+    @require(
+        lambda arguments_by_name:
+        all(
+            find_first_field_list(body) is None
+            for body in arguments_by_name.values()
+        ),
+        "Argument descriptions expected without field lists"
+    )
+    @require(
+        lambda returns:
+        not (returns is not None)
+        or find_first_field_list(returns) is None,
+        "Return value description expected without field lists"
+    )
+    # fmt: on
+    def __init__(
+        self,
+        summary: docutils.nodes.paragraph,
+        remarks: Sequence[docutils.nodes.Element],
+        arguments_by_name: OrderedDict[Identifier, docutils.nodes.field_body],
+        returns: Optional[docutils.nodes.field_body],
+        parsed: parse.Description,
+    ) -> None:
+        """Initialize with the given values."""
+        SummaryRemarksDescription.__init__(
+            self, summary=summary, remarks=remarks, parsed=parsed
+        )
+        self.arguments_by_name = arguments_by_name
+        self.returns = returns
+
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return f"<{_MODULE_NAME}.{self.__class__.__name__} at 0x{id(self):x}>"
+
+
+# endregion
 
 
 class Property:
@@ -209,7 +383,7 @@ class Property:
     type_annotation: Final[TypeAnnotationUnion]
 
     #: Description of the property, if any
-    description: Final[Optional[Description]]
+    description: Final[Optional[PropertyDescription]]
 
     #: The original class where this property is specified.
     #: We stack all the properties over the ancestors, so using ``specified_for``
@@ -224,7 +398,7 @@ class Property:
         self,
         name: Identifier,
         type_annotation: TypeAnnotationUnion,
-        description: Optional[Description],
+        description: Optional[PropertyDescription],
         specified_for: "Class",
         parsed: parse.Property,
     ) -> None:
@@ -335,6 +509,29 @@ class Serialization:
         self.with_model_type = with_model_type
 
 
+class ReferenceInTheBook:
+    """Represent the information indicated in the ``reference_in_the_book`` marker."""
+
+    #: Section number
+    section: Final[Tuple[int, ...]]
+
+    #: Index in the section so that the classes can be sorted deterministically
+    index: Final[int]
+
+    #: URL Fragment of the section
+    #:
+    #: The literal ``#`` needs to be prepended and the fragment needs to be URL-encoded.
+    fragment: Final[Optional[str]]
+
+    def __init__(
+        self, section: Tuple[int, ...], index: int, fragment: Optional[str]
+    ) -> None:
+        """Initialize with the given values."""
+        self.section = section
+        self.index = index
+        self.fragment = fragment
+
+
 class Invariant:
     """Represent an invariant of a class."""
 
@@ -456,7 +653,7 @@ class SignatureLike(DBC):
     returns: Final[Optional[TypeAnnotationUnion]]
 
     #: Description of the signature-like, if any
-    description: Final[Optional[Description]]
+    description: Final[Optional[SignatureDescription]]
 
     #: List of contracts of the signature-like. The contracts are stacked from the
     #: ancestors.
@@ -506,7 +703,7 @@ class SignatureLike(DBC):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         parsed: Optional[parse.Method],
     ) -> None:
@@ -577,7 +774,7 @@ class Method(SignatureLike):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         parsed: parse.Method,
     ) -> None:
@@ -645,7 +842,7 @@ class UnderstoodMethod(Method):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         body: Sequence[parse_tree.Node],
         parsed: parse.Method,
@@ -689,7 +886,7 @@ class Constructor(SignatureLike):
         is_implementation_specific: bool,
         arguments: Sequence[Argument],
         contracts: Contracts,
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         statements: Sequence[construction.AssignArgument],
         parsed: Optional[parse.Method],
     ) -> None:
@@ -719,8 +916,8 @@ class EnumerationLiteral:
     def __init__(
         self,
         name: Identifier,
-        value: Identifier,
-        description: Optional[Description],
+        value: str,
+        description: Optional[EnumerationLiteralDescription],
         parsed: parse.EnumerationLiteral,
     ) -> None:
         self.name = name
@@ -757,8 +954,11 @@ class Enumeration:
     #: this is akin to inheritance for enumerations
     is_superset_of: Final[Sequence["Enumeration"]]
 
+    #: Reference to the original specs
+    reference_in_the_book: Final[Optional[ReferenceInTheBook]]
+
     #: Description of the enumeration, if any
-    description: Final[Optional[Description]]
+    description: Final[Optional[SymbolDescription]]
 
     #: Map literals by their identifiers
     literals_by_name: Final[Mapping[str, EnumerationLiteral]]
@@ -771,12 +971,14 @@ class Enumeration:
         name: Identifier,
         literals: Sequence[EnumerationLiteral],
         is_superset_of: Sequence["Enumeration"],
-        description: Optional[Description],
+        reference_in_the_book: Optional[ReferenceInTheBook],
+        description: Optional[SymbolDescription],
         parsed: parse.Enumeration,
     ) -> None:
         self.name = name
         self.literals = literals
         self.is_superset_of = is_superset_of
+        self.reference_in_the_book = reference_in_the_book
         self.description = description
         self.parsed = parsed
 
@@ -786,6 +988,12 @@ class Enumeration:
 
         self.literal_id_set = frozenset(id(literal) for literal in literals)
 
+    def __repr__(self) -> str:
+        """Represent the instance as a string for easier debugging."""
+        return (
+            f"<{_MODULE_NAME}.{self.__class__.__name__} {self.name} at 0x{id(self):x}>"
+        )
+
 
 class ConstrainedPrimitive:
     """Represent a primitive type constrained by one or more invariants."""
@@ -838,11 +1046,33 @@ class ConstrainedPrimitive:
     #: for each implementation target
     is_implementation_specific: Final[bool]
 
-    #: List of class invariants
-    invariants: Final[Sequence[Invariant]]
+    # region Invariants
+
+    # NOTE (mristin, 2022-03-19):
+    # We have to decorate invariants with ``@property`` so that the translation code
+    # is forced to use ``_set_invariants``.
+
+    _invariants: Sequence[Invariant]
+
+    @property
+    def invariants(self) -> Sequence[Invariant]:
+        """List invariants of the class."""
+        return self._invariants
+
+    _invariant_id_set: FrozenSet[int]
+
+    @property
+    def invariant_id_set(self) -> FrozenSet[int]:
+        """Collect IDs (with :py:func:`id`) of the invariant objects in a set."""
+        return self._invariant_id_set
+
+    # endregion
+
+    #: Reference to the original specs
+    reference_in_the_book: Final[Optional[ReferenceInTheBook]]
 
     #: Description of the class
-    description: Final[Optional[Description]]
+    description: Final[Optional[SymbolDescription]]
 
     #: Relation to the class from the parse stage
     parsed: parse.Class
@@ -881,7 +1111,8 @@ class ConstrainedPrimitive:
         constrainee: PrimitiveType,
         is_implementation_specific: bool,
         invariants: Sequence[Invariant],
-        description: Optional[Description],
+        reference_in_the_book: Optional[ReferenceInTheBook],
+        description: Optional[SymbolDescription],
         parsed: parse.Class,
     ) -> None:
         self.name = name
@@ -889,12 +1120,11 @@ class ConstrainedPrimitive:
         self._set_descendants(descendants)
         self.constrainee = constrainee
         self.is_implementation_specific = is_implementation_specific
-        self.invariants = invariants
+        self._set_invariants(invariants)
+        self.reference_in_the_book = reference_in_the_book
         self.description = description
         self.parsed = parsed
 
-        self.invariant_id_set = frozenset(id(inv) for inv in self.invariants)
-
     def _set_descendants(self, descendants: Sequence["ConstrainedPrimitive"]) -> None:
         """
         Set the descendants in the constrained primitive.
@@ -905,6 +1135,15 @@ class ConstrainedPrimitive:
             id(descendant) for descendant in descendants
         )
 
+    def _set_invariants(self, invariants: Sequence[Invariant]) -> None:
+        """
+        Set the invariants in the class.
+
+        This method is expected to be called only during the translation phase.
+        """
+        self._invariants = invariants
+        self._invariant_id_set = frozenset(id(invariant) for invariant in invariants)
+
     def __repr__(self) -> str:
         """Represent the instance as a string for easier debugging."""
         return (
@@ -990,52 +1229,99 @@ class Class(DBC):
 
     # endregion
 
-    #: List of properties of the class
-    properties: Final[Sequence[Property]]
+    # region Properties
 
-    #: List of methods of the class. The methods are strictly non-static and non-class
-    #: (in the Python sense of the terms).
-    methods: Final[Sequence[Method]]
+    # NOTE (mristin, 2022-03-19):
+    # We have to decorate properties with ``@property`` so that the translation code
+    # is forced to use ``_set_properties``.
+
+    _properties: Sequence[Property]
+
+    @property
+    def properties(self) -> Sequence[Property]:
+        """Return list of properties of the class."""
+        return self._properties
+
+    _properties_by_name: Mapping[Identifier, Property]
+
+    @property
+    def properties_by_name(self) -> Mapping[Identifier, Property]:
+        """Map all properties by their names."""
+        return self._properties_by_name
+
+    _property_id_set: FrozenSet[int]
+
+    @property
+    def property_id_set(self) -> FrozenSet[int]:
+        """Collect IDs (with :py:func:`id`) of the property objects in a set."""
+        return self._property_id_set
+
+    # endregion
+
+    # region Methods
+
+    # NOTE (mristin, 2022-03-19):
+    # We have to decorate methods with ``@property`` so that the translation code
+    # is forced to use ``_set_methods``.
+
+    _methods: Sequence["MethodUnion"]
+
+    @property
+    def methods(self) -> Sequence["MethodUnion"]:
+        """
+        List methods of the class.
+
+        The methods are strictly non-static and non-class (in the Python sense of
+        the terms).
+        """
+        return self._methods
+
+    _methods_by_name: Mapping[Identifier, "MethodUnion"]
+
+    @property
+    def methods_by_name(self) -> Mapping[Identifier, "MethodUnion"]:
+        """Map all methods by their names."""
+        return self._methods_by_name
+
+    # endregion
 
     #: Constructor specification of the class
     constructor: Final[Constructor]
 
-    #: List of class invariants
-    invariants: Final[Sequence[Invariant]]
+    # region Invariants
+
+    # NOTE (mristin, 2022-03-19):
+    # We have to decorate invariants with ``@property`` so that the translation code
+    # is forced to use ``_set_invariants``.
+
+    _invariants: Sequence[Invariant]
+
+    @property
+    def invariants(self) -> Sequence[Invariant]:
+        """List invariants of the class."""
+        return self._invariants
+
+    _invariant_id_set: FrozenSet[int]
+
+    @property
+    def invariant_id_set(self) -> FrozenSet[int]:
+        """Collect IDs (with :py:func:`id`) of the invariant objects in a set."""
+        return self._invariant_id_set
+
+    # endregion
 
     #: Particular serialization settings for this class
     serialization: Final[Serialization]
 
+    #: Reference to the original specs
+    reference_in_the_book: Final[Optional[ReferenceInTheBook]]
+
     #: Description of the class
-    description: Final[Optional[Description]]
+    description: Final[Optional[SymbolDescription]]
 
     #: Relation to the class from the parse stage
     parsed: Final[parse.Class]
 
-    #: Map all properties by their identifiers to the corresponding objects
-    properties_by_name: Final[Mapping[Identifier, Property]]
-
-    #: Collect IDs (with :py:func:`id`) of the property objects in a set
-    property_id_set: Final[FrozenSet[int]]
-
-    #: Map all methods by their identifiers to the corresponding objects
-    methods_by_name: Final[Mapping[Identifier, Method]]
-
-    #: Collect IDs (with :py:func:`id`) of the invariant objects in a set
-    invariant_id_set: Final[FrozenSet[int]]
-
-    # fmt: off
-    @require(
-        lambda properties:
-        len(properties) == len(set(prop.name for prop in properties)),
-        "No duplicate properties"
-    )
-    @require(
-        lambda methods:
-        len(methods) == len(set(method.name for method in methods)),
-        "No duplicate methods"
-    )
-    # fmt: on
     def __init__(
         self,
         name: Identifier,
@@ -1044,36 +1330,29 @@ class Class(DBC):
         descendants: Sequence["ClassUnion"],
         is_implementation_specific: bool,
         properties: Sequence[Property],
-        methods: Sequence[Method],
+        methods: Sequence["MethodUnion"],
         constructor: Constructor,
         invariants: Sequence[Invariant],
         serialization: Serialization,
-        description: Optional[Description],
+        reference_in_the_book: Optional[ReferenceInTheBook],
+        description: Optional[SymbolDescription],
         parsed: parse.Class,
     ) -> None:
         """Initialize with the given values."""
         self.name = name
         self._set_inheritances(inheritances)
-        self._set_descendants(descendants)
-
         self.interface = interface
+        self._set_descendants(descendants)
         self.is_implementation_specific = is_implementation_specific
-        self.properties = properties
-        self.methods = methods
+        self._set_properties(properties)
+        self._set_methods(methods)
         self.constructor = constructor
-        self.invariants = invariants
+        self._set_invariants(invariants)
         self.serialization = serialization
+        self.reference_in_the_book = reference_in_the_book
         self.description = description
         self.parsed = parsed
 
-        self.properties_by_name = {prop.name: prop for prop in self.properties}
-
-        self.property_id_set = frozenset(id(prop) for prop in self.properties)
-
-        self.methods_by_name = {method.name: method for method in self.methods}
-
-        self.invariant_id_set = frozenset(id(inv) for inv in self.invariants)
-
     # fmt: off
     @require(
         lambda inheritances:
@@ -1109,6 +1388,48 @@ class Class(DBC):
             if isinstance(descendant, ConcreteClass)
         ]
 
+    # fmt: off
+    @require(
+        lambda properties:
+        len(properties) == len(set(prop.name for prop in properties)),
+        "No duplicate properties"
+    )
+    # fmt: on
+    def _set_properties(self, properties: Sequence[Property]) -> None:
+        """
+        Set the properties in the class.
+
+        This method is expected to be called only during the translation phase.
+        """
+        self._properties = properties
+        self._properties_by_name = {prop.name: prop for prop in properties}
+        self._property_id_set = frozenset(id(prop) for prop in properties)
+
+    # fmt: off
+    @require(
+        lambda methods:
+        len(methods) == len(set(method.name for method in methods)),
+        "No duplicate methods"
+    )
+    # fmt: on
+    def _set_methods(self, methods: Sequence["MethodUnion"]) -> None:
+        """
+        Set the methods in the class.
+
+        This method is expected to be called only during the translation phase.
+        """
+        self._methods = methods
+        self._methods_by_name = {method.name: method for method in methods}
+
+    def _set_invariants(self, invariants: Sequence[Invariant]) -> None:
+        """
+        Set the invariants in the class.
+
+        This method is expected to be called only during the translation phase.
+        """
+        self._invariants = invariants
+        self._invariant_id_set = frozenset(id(invariant) for invariant in invariants)
+
     @abc.abstractmethod
     def __repr__(self) -> str:
         # Signal that this is a purely abstract class.
@@ -1141,11 +1462,12 @@ class AbstractClass(Class):
         descendants: Sequence["ClassUnion"],
         is_implementation_specific: bool,
         properties: Sequence[Property],
-        methods: Sequence[Method],
+        methods: Sequence["MethodUnion"],
         constructor: Constructor,
         invariants: Sequence[Invariant],
         serialization: Serialization,
-        description: Optional[Description],
+        reference_in_the_book: Optional[ReferenceInTheBook],
+        description: Optional[SymbolDescription],
         parsed: parse.Class,
     ) -> None:
         """Initialize with the given values."""
@@ -1161,6 +1483,7 @@ class AbstractClass(Class):
             constructor=constructor,
             invariants=invariants,
             serialization=serialization,
+            reference_in_the_book=reference_in_the_book,
             description=description,
             parsed=parsed,
         )
@@ -1207,7 +1530,7 @@ class Verification(SignatureLike):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         parsed: parse.Method,
     ) -> None:
@@ -1236,7 +1559,7 @@ class ImplementationSpecificVerification(Verification):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         parsed: parse.Method,
     ) -> None:
@@ -1292,7 +1615,7 @@ class PatternVerification(Verification):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         pattern: str,
         parsed: parse.UnderstoodMethod,
@@ -1325,7 +1648,7 @@ class Signature(SignatureLike):
         name: Identifier,
         arguments: Sequence[Argument],
         returns: Optional[TypeAnnotationUnion],
-        description: Optional[Description],
+        description: Optional[SignatureDescription],
         contracts: Contracts,
         parsed: parse.Method,
     ) -> None:
@@ -1380,7 +1703,7 @@ class Interface:
     signatures: Final[Sequence[Signature]]
 
     #: Description of the interface, taken from class
-    description: Final[Optional[Description]]
+    description: Final[Optional[SymbolDescription]]
 
     #: Relation to the class from the parse stage
     parsed: Final[parse.Class]
@@ -1446,7 +1769,7 @@ class MetaModel:
     """Collect information about the underlying meta-model."""
 
     #: Description of the meta-model extracted from the docstring
-    description: Final[Optional[Description]]
+    description: Final[Optional[MetaModelDescription]]
 
     #: Specify the URL of the book that the meta-model is based on
     book_url: Final[str]
@@ -1455,7 +1778,10 @@ class MetaModel:
     book_version: Final[str]
 
     def __init__(
-        self, book_url: str, book_version: str, description: Optional[Description]
+        self,
+        book_url: str,
+        book_version: str,
+        description: Optional[MetaModelDescription],
     ) -> None:
         self.book_url = book_url
         self.book_version = book_version
@@ -1468,15 +1794,15 @@ class SymbolTable:
     #: List of all symbols that we need for the code generation
     symbols: Final[Sequence["Symbol"]]
 
+    #: List of all the symbols, topologically sorted by inheritance
+    symbols_topologically_sorted: Final[Sequence["Symbol"]]
+
     #: List of all functions used in the verification
     verification_functions: Final[Sequence["VerificationUnion"]]
 
     #: Map verification functions by their name
     verification_functions_by_name: Final[Mapping[Identifier, "VerificationUnion"]]
 
-    #: Type to be used to represent a ``Ref[T]``
-    ref_association: Final["ClassUnion"]
-
     #: Additional information about the source meta-model
     meta_model: Final[MetaModel]
 
@@ -1490,6 +1816,17 @@ class SymbolTable:
         )[1],
         "Symbol names unique",
     )
+    @require(
+        lambda symbols, symbols_topologically_sorted:
+        set(
+            id(symbol)
+            for symbol in symbols
+            if not isinstance(symbol, Enumeration)
+        )
+        == set(id(symbol) for symbol in symbols_topologically_sorted),
+        "Only maybe the order differs between the symbols and "
+        "symbols_topologically_sorted"
+    )
     @ensure(
         lambda self:
         all(
@@ -1515,14 +1852,14 @@ class SymbolTable:
     def __init__(
         self,
         symbols: Sequence["Symbol"],
+        symbols_topologically_sorted: Sequence["SymbolExceptEnumeration"],
         verification_functions: Sequence["VerificationUnion"],
-        ref_association: "ClassUnion",
         meta_model: MetaModel,
     ) -> None:
         """Initialize with the given values and map symbols to name."""
         self.symbols = symbols
+        self.symbols_topologically_sorted = symbols_topologically_sorted
         self.verification_functions = verification_functions
-        self.ref_association = ref_association
         self.meta_model = meta_model
 
         self.verification_functions_by_name = {
@@ -1552,7 +1889,7 @@ class SymbolTable:
 
 
 def map_descendability(
-    type_annotation: TypeAnnotationUnion, ref_association: Class
+    type_annotation: TypeAnnotationUnion,
 ) -> MutableMapping[TypeAnnotationUnion, bool]:
     """
     Map the type annotation recursively by the descendability.
@@ -1565,9 +1902,6 @@ def map_descendability(
 
     The mapping is a form of caching. Otherwise, the time complexity would be quadratic
     if we queried at each type annotation subscript.
-
-    The ``ref_association`` indicates which symbol to use for representing references
-    within an AAS.
     """
     mapping = dict()  # type: MutableMapping[TypeAnnotationUnion, bool]
 
@@ -1602,13 +1936,6 @@ def map_descendability(
             mapping[a_type_annotation] = result
             return result
 
-        elif isinstance(a_type_annotation, RefTypeAnnotation):
-            assert isinstance(
-                ref_association, Class
-            ), "Explicit assumption for descendability"
-            mapping[a_type_annotation] = True
-            return True
-
         else:
             assert_never(a_type_annotation)
 
@@ -1628,140 +1955,9 @@ class _ConstructorArgumentOfClass:
         self.cls = cls
 
 
-def make_union_of_constructor_arguments(
-    interface: Interface,
-) -> Tuple[Optional[OrderedDict[Identifier, TypeAnnotationUnion]], Optional[Error]]:
+def collect_ids_of_symbols_in_properties(symbol_table: SymbolTable) -> Set[int]:
     """
-    Make a union of all the constructor arguments over all the implementing classes.
-
-    This union is necessary, for example, when you need to de-serialize an object, but
-    you are not yet sure which concrete type it has. Hence you need to be prepared to
-    de-serialize a yet-unknown *subset* of the properties of *this* union when you start
-    de-serializing an object of type ``interface``.
-    """
-    errors = []  # type: List[Error]
-
-    arg_union = (
-        collections.OrderedDict()
-    )  # type: OrderedDict[Identifier, List[_ConstructorArgumentOfClass]]
-
-    # region Collect
-
-    for cls_arg, arg in (
-        (implementer, arg)
-        for implementer in interface.implementers
-        for arg in implementer.constructor.arguments
-    ):
-        lst = arg_union.get(arg.name, None)
-        if lst is None:
-            lst = []
-            arg_union[arg.name] = lst
-
-        lst.append(_ConstructorArgumentOfClass(arg=arg, cls=cls_arg))
-
-    # endregion
-
-    # region Resolve
-
-    resolution = (
-        collections.OrderedDict()
-    )  # type: OrderedDict[Identifier, TypeAnnotationUnion]
-
-    for args_of_clses in arg_union.values():
-        # NOTE (mristin, 2021-12-19):
-        # We have to check that the arguments share the same type. We have to allow
-        # that the non-nullability constraint is strengthened since implementers can
-        # strengthen the invariants.
-
-        # This is the argument that defines the current resolved type.
-        defining_arg = None  # type: Optional[_ConstructorArgumentOfClass]
-
-        def normalize_type_annotation(
-            type_anno: TypeAnnotationUnion,
-        ) -> TypeAnnotationUnion:
-            """Normalize the type annotation by removing prefix ``Optional``'s."""
-            while isinstance(type_anno, OptionalTypeAnnotation):
-                type_anno = type_anno.value
-
-            return type_anno
-
-        for arg_of_cls in args_of_clses:
-            # Set if the resolution not possible for the current argument
-            inconsistent = False
-
-            if defining_arg is None:
-                defining_arg = arg_of_cls
-            else:
-                if type_annotations_equal(
-                    defining_arg.arg.type_annotation, arg_of_cls.arg.type_annotation
-                ):
-                    # Leave the previous argument the defining one
-                    continue
-                else:
-                    if type_annotations_equal(
-                        normalize_type_annotation(defining_arg.arg.type_annotation),
-                        normalize_type_annotation(arg_of_cls.arg.type_annotation),
-                    ):
-                        # NOTE (mristin, 2021-12-19):
-                        # The type with ``Optional`` will win the resolution so that
-                        # we allow for strengthening of invariants.
-
-                        if isinstance(
-                            defining_arg.arg.type_annotation, OptionalTypeAnnotation
-                        ):
-                            # The defining argument wins.
-                            continue
-                        elif isinstance(
-                            arg_of_cls.arg.type_annotation, OptionalTypeAnnotation
-                        ):
-                            # The current argument wins.
-                            defining_arg = arg_of_cls
-                        else:
-                            raise AssertionError(
-                                f"Unexpected case: "
-                                f"{arg_of_cls.arg.type_annotation=}, "
-                                f"{defining_arg.arg.type_annotation=}"
-                            )
-                    else:
-                        inconsistent = True
-
-            if inconsistent:
-                errors.append(
-                    Error(
-                        arg_of_cls.cls.parsed.node,
-                        f"The constructor argument {defining_arg.arg.name!r} "
-                        f"of the class {defining_arg.cls.name!r} "
-                        f"has inconsistent type ({defining_arg.arg.type_annotation}) "
-                        f"with the constructor argument {arg_of_cls.arg.name!r} "
-                        f"of the class {arg_of_cls.cls.name!r} "
-                        f"({arg_of_cls.arg.type_annotation}. "
-                        f"This is a blocker for generating efficient code for "
-                        f"JSON de-serialization of the interface {interface.name!r} "
-                        f"(which both {defining_arg.cls.name!r} and "
-                        f"{arg_of_cls.cls.name!r} implement).",
-                    )
-                )
-
-            # endregion
-
-        assert defining_arg is not None, "Expected to be set before"
-
-        resolution[defining_arg.arg.name] = defining_arg.arg.type_annotation
-
-    if len(errors) > 0:
-        return None, Error(
-            interface.parsed.node,
-            f"Failed to make a union of constructor arguments "
-            f"over all the concrete classes of the class {interface.name!r}",
-            errors,
-        )
-
-    return resolution, None
-
-
-def collect_ids_of_classes_in_properties(symbol_table: SymbolTable) -> Set[int]:
-    """
-    Collect the IDs of the classes occurring in type annotations of the properties.
+    Collect the IDs of the symbols occurring in type annotations of the properties.
 
     The IDs refer to IDs of the Python objects in this context.
     """
@@ -1784,8 +1980,6 @@ def collect_ids_of_classes_in_properties(symbol_table: SymbolTable) -> Set[int]:
                         type_anno = type_anno.value
                     elif isinstance(type_anno, ListTypeAnnotation):
                         type_anno = type_anno.items
-                    elif isinstance(type_anno, RefTypeAnnotation):
-                        type_anno = type_anno.value
                     elif isinstance(type_anno, PrimitiveTypeAnnotation):
                         break
                     elif isinstance(type_anno, OurTypeAnnotation):
@@ -1802,10 +1996,29 @@ def collect_ids_of_classes_in_properties(symbol_table: SymbolTable) -> Set[int]:
     return result
 
 
+DescriptionUnion = Union[
+    MetaModelDescription,
+    SymbolDescription,
+    PropertyDescription,
+    EnumerationLiteralDescription,
+    SignatureDescription,
+]
+assert_union_of_descendants_exhaustive(
+    union=DescriptionUnion, base_class=SummaryRemarksDescription
+)
+
 ClassUnion = Union[AbstractClass, ConcreteClass]
 assert_union_of_descendants_exhaustive(union=ClassUnion, base_class=Class)
 
+MethodUnion = Union[UnderstoodMethod, ImplementationSpecificMethod]
+assert_union_of_descendants_exhaustive(union=MethodUnion, base_class=Method)
+
 Symbol = Union[Enumeration, ConstrainedPrimitive, ClassUnion]
 
+SymbolExceptEnumeration = Union[ConstrainedPrimitive, ClassUnion]
+assert_union_without_excluded(
+    original_union=Symbol, subset_union=SymbolExceptEnumeration, excluded=[Enumeration]
+)
+
 VerificationUnion = Union[ImplementationSpecificVerification, PatternVerification]
 assert_union_of_descendants_exhaustive(union=VerificationUnion, base_class=Verification)
