@@ -77,6 +77,9 @@ def _define_primitive_type(
         if len_constraint.max_value is not None:
             type_definition["maxLength"] = len_constraint.max_value
 
+    if primitive_type is intermediate.PrimitiveType.BYTEARRAY:
+        type_definition["contentEncoding"] = "base64"
+
     if (
         primitive_type == intermediate.PrimitiveType.STR
         and pattern_constraints is not None
@@ -100,16 +103,10 @@ def _define_primitive_type(
 @ensure(lambda result: (result[0] is not None) ^ (result[1] is not None))
 def _define_type(
     type_annotation: intermediate.TypeAnnotation,
-    ref_association: intermediate.ClassUnion,
     len_constraint: Optional[infer_for_schema.LenConstraint],
     pattern_constraints: Optional[Sequence[infer_for_schema.PatternConstraint]],
 ) -> Tuple[Optional[MutableMapping[str, Any]], Optional[Error]]:
-    """
-    Generate the type definition for ``type_annotation``.
-
-    The ``ref_association`` indicates which symbol to use for representing references
-    within an AAS.
-    """
+    """Generate the type definition for ``type_annotation``."""
     if isinstance(type_annotation, intermediate.PrimitiveTypeAnnotation):
         type_definition = _define_primitive_type(
             primitive_type=type_annotation.a_type,
@@ -129,51 +126,25 @@ def _define_type(
             )
 
         elif isinstance(type_annotation.symbol, intermediate.ConstrainedPrimitive):
-            type_definition = collections.OrderedDict(
-                [("$ref", f"#/definitions/{model_type}")]
+            # NOTE (mristin, 2022-02-11):
+            # We in-line the constraints from the constrained primitives directly
+            # in the properties. We do not want to introduce separate definitions
+            # for them as that would make it more difficult for downstream code
+            # generators to generate meaningful code (*e.g.*, code generators for
+            # OpenAPI3).
+
+            type_definition = _define_primitive_type(
+                primitive_type=type_annotation.symbol.constrainee,
+                len_constraint=len_constraint,
+                pattern_constraints=pattern_constraints,
             )
 
-            if (
-                type_annotation.symbol.constrainee
-                in (
-                    intermediate.PrimitiveType.STR,
-                    intermediate.PrimitiveType.BYTEARRAY,
-                )
-                and len_constraint is not None
-            ):
-                if len_constraint.min_value is not None:
-                    type_definition["minLength"] = len_constraint.min_value
-
-                if len_constraint.max_value is not None:
-                    type_definition["maxLength"] = len_constraint.max_value
-
-            if (
-                type_annotation.symbol.constrainee is intermediate.PrimitiveType.STR
-                and pattern_constraints is not None
-                and len(pattern_constraints) > 0
-            ):
-                if len(pattern_constraints) == 1:
-                    type_definition["pattern"] = pattern_constraints[0].pattern
-                else:
-                    all_of = [type_definition]  # type: List[MutableMapping[str, Any]]
-
-                    for pattern_constraint in pattern_constraints:
-                        all_of.append(
-                            collections.OrderedDict(
-                                [("pattern", pattern_constraint.pattern)]
-                            )
-                        )
-
-                    type_definition = collections.OrderedDict([("allOf", all_of)])
-
             return type_definition, None
 
         elif isinstance(type_annotation.symbol, intermediate.Class):
             if type_annotation.symbol.interface is not None:
                 return (
-                    collections.OrderedDict(
-                        [("$ref", f"#/definitions/{model_type}_abstract")]
-                    ),
+                    collections.OrderedDict([("$ref", f"#/definitions/{model_type}")]),
                     None,
                 )
             else:
@@ -194,7 +165,6 @@ def _define_type(
 
         items_type_definition, items_error = _define_type(
             type_annotation=type_annotation.items,
-            ref_association=ref_association,
             len_constraint=None,
             pattern_constraints=None,
         )
@@ -235,11 +205,6 @@ def _define_type(
             f"{type_annotation=}"
         )
 
-    elif isinstance(type_annotation, intermediate.RefTypeAnnotation):
-        model_type = naming.json_model_type(ref_association.name)
-
-        return collections.OrderedDict([("$ref", f"#/definitions/{model_type}")]), None
-
     else:
         raise NotImplementedError(
             f"(mristin, 2021-11-10):\n"
@@ -251,71 +216,6 @@ def _define_type(
         )
 
 
-def _define_for_constrained_primitive(
-    constrained_primitive: intermediate.ConstrainedPrimitive,
-    pattern_verifications_by_name: infer_for_schema.PatternVerificationsByName,
-) -> Tuple[Optional[MutableMapping[str, Any]], Optional[List[Error]]]:
-    """
-    Generate the JSON definitions based on the ``constrained_primitive``.
-
-    The list of definitions is to be *extended* with the resulting mapping.
-    """
-    all_of = []  # type: List[MutableMapping[str, Any]]
-
-    # region Inheritance
-
-    for inheritance in constrained_primitive.inheritances:
-        all_of.append(
-            {"$ref": f"#/definitions/{naming.json_model_type(inheritance.name)}"}
-        )
-
-    # endregion
-
-    # region Constraints
-
-    errors = []  # type: List[Error]
-
-    (
-        len_constraint,
-        len_constraint_errors,
-    ) = infer_for_schema.infer_len_constraint_of_self(
-        constrained_primitive=constrained_primitive
-    )
-
-    if len_constraint_errors is not None:
-        errors.extend(len_constraint_errors)
-
-    if len(errors) > 0:
-        return None, errors
-
-    assert len_constraint is not None
-
-    pattern_constraints = infer_for_schema.infer_patterns_on_self(
-        constrained_primitive=constrained_primitive,
-        pattern_verifications_by_name=pattern_verifications_by_name,
-    )
-
-    type_definition = _define_primitive_type(
-        primitive_type=constrained_primitive.constrainee,
-        len_constraint=len_constraint,
-        pattern_constraints=pattern_constraints,
-    )
-
-    all_of.append(type_definition)
-
-    # endregion
-
-    model_type = naming.json_model_type(constrained_primitive.name)
-
-    result = collections.OrderedDict()  # type: MutableMapping[str, Any]
-    assert (
-        len(all_of) >= 1
-    ), "At least the type definition for the primitive type expected"
-    result[model_type] = {"allOf": all_of} if len(all_of) > 1 else all_of[0]
-
-    return result, None
-
-
 # fmt: off
 @ensure(
     lambda result:
@@ -324,40 +224,15 @@ def _define_for_constrained_primitive(
 # fmt: on
 def _define_properties_and_required(
     cls: intermediate.Class,
-    ref_association: intermediate.ClassUnion,
-    pattern_verifications_by_name: infer_for_schema.PatternVerificationsByName,
+    constraints_by_property: infer_for_schema.ConstraintsByProperty,
 ) -> Tuple[
     Optional[MutableMapping[str, Any]],
     Optional[List[Identifier]],
     Optional[List[Error]],
 ]:
-    """
-    Define the ``properties`` and ``required`` part for the given class ``cls``.
-
-    The ``ref_association`` indicates which symbol to use for representing references
-    within an AAS.
-    """
+    """Define the ``properties`` and ``required`` part for the given class ``cls``."""
     errors = []  # type: List[Error]
 
-    (
-        len_constraints_by_property,
-        len_constraints_errors,
-    ) = infer_for_schema.infer_len_constraints_by_class_properties(cls=cls)
-
-    if len_constraints_errors is not None:
-        errors.extend(len_constraints_errors)
-
-    if len(errors) > 0:
-        return None, None, errors
-
-    assert len_constraints_by_property is not None
-
-    pattern_constraints_by_property = (
-        infer_for_schema.infer_patterns_by_class_properties(
-            cls=cls, pattern_verifications_by_name=pattern_verifications_by_name
-        )
-    )
-
     properties = collections.OrderedDict()  # type: MutableMapping[str, Any]
     required = []  # type: List[Identifier]
 
@@ -367,8 +242,13 @@ def _define_properties_and_required(
 
         prop_name = naming.json_property(prop.name)
 
-        len_constraint = len_constraints_by_property.get(prop, None)
-        pattern_constraints = pattern_constraints_by_property.get(prop, None)
+        len_constraint = constraints_by_property.len_constraints_by_property.get(
+            prop, None
+        )
+
+        pattern_constraints = constraints_by_property.patterns_by_property.get(
+            prop, None
+        )
 
         # noinspection PyUnusedLocal
         type_anno = None  # type: Optional[intermediate.TypeAnnotation]
@@ -382,7 +262,6 @@ def _define_properties_and_required(
         assert type_anno is not None
         type_definition, error = _define_type(
             type_annotation=type_anno,
-            ref_association=ref_association,
             len_constraint=len_constraint,
             pattern_constraints=pattern_constraints,
         )
@@ -393,6 +272,12 @@ def _define_properties_and_required(
             assert type_definition is not None
             properties[prop_name] = type_definition
 
+    if cls.serialization.with_model_type and not any(
+        inheritance.serialization.with_model_type for inheritance in cls.inheritances
+    ):
+        properties["modelType"] = {"$ref": "#/definitions/ModelType"}
+        required.append(Identifier("modelType"))
+
     if len(errors) > 0:
         return None, None, errors
 
@@ -401,18 +286,14 @@ def _define_properties_and_required(
 
 @ensure(lambda result: (result[0] is not None) ^ (result[1] is not None))
 def _define_for_class(
-    cls: intermediate.Class,
-    ref_association: intermediate.ClassUnion,
+    cls: intermediate.ClassUnion,
     ids_of_classes_in_properties: Set[int],
-    pattern_verifications_by_name: infer_for_schema.PatternVerificationsByName,
+    constraints_by_property: infer_for_schema.ConstraintsByProperty,
 ) -> Tuple[Optional[MutableMapping[str, Any]], Optional[List[Error]]]:
     """
     Generate the JSON definitions based on the class ``cls``.
 
     The list of definitions is to be *extended* with the resulting mapping.
-
-    The ``ref_association`` indicates which symbol to use for representing references
-    within an AAS.
     """
     all_of = []  # type: List[MutableMapping[str, Any]]
 
@@ -431,8 +312,7 @@ def _define_for_class(
 
     properties, required, properties_error = _define_properties_and_required(
         cls=cls,
-        ref_association=ref_association,
-        pattern_verifications_by_name=pattern_verifications_by_name,
+        constraints_by_property=constraints_by_property,
     )
 
     if properties_error is not None:
@@ -444,10 +324,12 @@ def _define_for_class(
     assert properties is not None
     assert required is not None
 
+    definition = collections.OrderedDict()  # type: MutableMapping[str, Any]
+    if len(cls.inheritances) == 0:
+        definition["type"] = "object"
+
     if len(properties) > 0:
-        definition = collections.OrderedDict(
-            [("type", "object"), ("properties", properties)]
-        )
+        definition["properties"] = properties
 
         if len(required) > 0:
             definition["required"] = required
@@ -459,26 +341,12 @@ def _define_for_class(
     model_type = naming.json_model_type(cls.name)
 
     result = collections.OrderedDict()  # type: MutableMapping[str, Any]
-    result[model_type] = {"allOf": all_of} if len(all_of) > 0 else {"type": "object"}
-
-    # region Define the abstract part
-
-    # NOTE (mristin, 2022-01-02):
-    # We generate the "*_abstract" definition of a class only if it is used to specify
-    # the type of one or more properties in the meta-model. Otherwise, we can ignore
-    # the abstract definition as it wouldn't be used during the validation.
-
-    if len(cls.concrete_descendants) > 0 and id(cls) in ids_of_classes_in_properties:
-        model_type_abstract = f"{model_type}_abstract"
-
-        any_of = [
-            {"$ref": f"#/definitions/{naming.json_model_type(descendant.name)}"}
-            for descendant in cls.concrete_descendants
-        ]  # type: List[MutableMapping[str, Any]]
-
-        result[model_type_abstract] = {"anyOf": any_of}
-
-    # endregion
+    if len(all_of) == 0:
+        result[model_type] = {"type": "object"}
+    elif len(all_of) == 1:
+        result[model_type] = all_of[0]
+    else:
+        result[model_type] = {"allOf": all_of}
 
     return result, None
 
@@ -527,11 +395,19 @@ def _generate(
 
     definitions = collections.OrderedDict()
 
-    pattern_verifications_by_name = infer_for_schema.map_pattern_verifications_by_name(
-        verifications=symbol_table.verification_functions
+    constraints_by_class, some_errors = infer_for_schema.infer_constraints_by_class(
+        symbol_table=symbol_table
     )
 
-    ids_of_classes_in_properties = intermediate.collect_ids_of_classes_in_properties(
+    if some_errors is not None:
+        errors.extend(some_errors)
+
+    if len(errors) > 0:
+        return None, errors
+
+    assert constraints_by_class is not None
+
+    ids_of_symbols_in_properties = intermediate.collect_ids_of_symbols_in_properties(
         symbol_table=symbol_table
     )
 
@@ -580,24 +456,28 @@ def _generate(
                 continue
         else:
             if isinstance(symbol, intermediate.Enumeration):
+                if id(symbol) not in ids_of_symbols_in_properties:
+                    continue
+
                 extension = _define_for_enumeration(enumeration=symbol)
 
             elif isinstance(symbol, intermediate.ConstrainedPrimitive):
-                extension, definition_errors = _define_for_constrained_primitive(
-                    constrained_primitive=symbol,
-                    pattern_verifications_by_name=pattern_verifications_by_name,
-                )
+                # NOTE (mristin, 2022-02-11):
+                # We in-line the constraints from the constrained primitives directly
+                # in the properties. We do not want to introduce separate definitions
+                # for them as that would make it more difficult for downstream code
+                # generators to generate meaningful code (*e.g.*, code generators for
+                # OpenAPI3).
 
-                if definition_errors is not None:
-                    errors.extend(definition_errors)
-                    continue
+                continue
 
-            elif isinstance(symbol, intermediate.Class):
+            elif isinstance(
+                symbol, (intermediate.AbstractClass, intermediate.ConcreteClass)
+            ):
                 extension, definition_errors = _define_for_class(
                     cls=symbol,
-                    ref_association=symbol_table.ref_association,
-                    ids_of_classes_in_properties=ids_of_classes_in_properties,
-                    pattern_verifications_by_name=pattern_verifications_by_name,
+                    ids_of_classes_in_properties=ids_of_symbols_in_properties,
+                    constraints_by_property=constraints_by_class[symbol],
                 )
 
                 if definition_errors is not None:
@@ -627,6 +507,34 @@ def _generate(
     if len(errors) > 0:
         return None, errors
 
+    model_types = [
+        naming.json_model_type(symbol.name)
+        for symbol in symbol_table.symbols
+        if (
+            isinstance(symbol, intermediate.ConcreteClass)
+            and symbol.serialization.with_model_type
+        )
+    ]  # type: List[Identifier]
+
+    definitions["ModelTypes"] = collections.OrderedDict(
+        [("type", "string"), ("enum", model_types)]
+    )
+
+    definitions["ModelType"] = collections.OrderedDict(
+        [
+            ("type", "object"),
+            (
+                "properties",
+                collections.OrderedDict(
+                    [
+                        ("name", {"$ref": "#/definitions/ModelTypes"}),
+                    ]
+                ),
+            ),
+            ("required", ["name"]),
+        ]
+    )
+
     schema["definitions"] = definitions
 
     return Stripped(json.dumps(schema, indent=2)), None
