@@ -11,6 +11,7 @@ from aas_core_codegen import parse, run, specific_implementations, intermediate
 from aas_core_codegen.common import LinenoColumner, assert_never
 import aas_core_codegen.csharp.main as csharp_main
 import aas_core_codegen.jsonschema.main as jsonschema_main
+import aas_core_codegen.rdf_shacl.main as rdf_shacl_main
 
 assert aas_core_codegen.__doc__ == __doc__
 
@@ -20,6 +21,7 @@ class Target(enum.Enum):
 
     CSHARP = "csharp"
     JSONSCHEMA = "jsonschema"
+    RDF_SHACL = "rdf_shacl"
 
 
 class Parameters:
@@ -179,6 +181,9 @@ def execute(params: Parameters, stdout: TextIO, stderr: TextIO) -> int:
             context=run_context, stdout=stdout, stderr=stderr
         )
 
+    elif params.target is Target.RDF_SHACL:
+        return rdf_shacl_main.execute(context=run_context, stdout=stdout, stderr=stderr)
+
     else:
         assert_never(params.target)
 
@@ -211,6 +216,17 @@ def main(prog: str) -> int:
         required=True,
         choices=[literal.value for literal in Target],
     )
+    parser.add_argument(
+        "--version", help="show the current version and exit", action="store_true"
+    )
+
+    # NOTE (mristin, 2022-01-14):
+    # The module ``argparse`` is not flexible enough to understand special options such
+    # as ``--version`` so we manually hard-wire.
+    if "--version" in sys.argv and "--help" not in sys.argv:
+        print(aas_core_codegen.__version__)
+        return 1
+
     args = parser.parse_args()
 
     target_to_str = {literal.value: literal for literal in Target}
