@@ -1,3 +1,5 @@
+# pylint: disable=missing-docstring
+
 import os
 import pathlib
 import textwrap
@@ -17,28 +19,28 @@ class Test_in_lining_of_constructor_statements(unittest.TestCase):
             @abstract
             class VeryAbstract:
                 some_property: int
-            
+
                 @require(lambda some_property: some_property > 0)
                 def __init__(self, some_property: int) -> None:
                     self.some_property = some_property
-            
-            
+
+
             @abstract
             class Abstract(VeryAbstract):
                 another_property: int
-            
+
                 @require(lambda another_property: another_property > 0)
                 def __init__(self, some_property: int, another_property: int) -> None:
                     VeryAbstract.__init__(self, some_property)
                     self.another_property = another_property
-                    
+
             class Concrete(Abstract):
                 yet_another_property: int
-            
+
                 @require(lambda yet_another_property: yet_another_property > 0)
                 def __init__(
-                        self, 
-                        some_property: int, 
+                        self,
+                        some_property: int,
                         another_property: int,
                         yet_another_property: int
                 ) -> None:
@@ -46,19 +48,15 @@ class Test_in_lining_of_constructor_statements(unittest.TestCase):
                     self.yet_another_property = yet_another_property
 
 
-            class Reference:
-                pass
-                
             __book_url__ = "dummy"
             __book_version__ = "dummy"
-            associate_ref_with(Reference)
             """
         )
 
         symbol_table, error = tests.common.translate_source_to_intermediate(
             source=source
         )
-        assert error is None, f"{error=}"
+        assert error is None, tests.common.most_underlying_messages(error)
 
         assert symbol_table is not None
 
@@ -79,22 +77,18 @@ class Test_parsing_docstrings(unittest.TestCase):
                 """
                 This is some documentation.
 
-                 * Nested reference :class:`.Some_class`
-                 """
-            
-            class Reference:
-                pass
-                
+                Nested reference :class:`.Some_class`
+                """
+
             __book_url__ = "dummy"
             __book_version__ = "dummy"
-            associate_ref_with(Reference)
             '''
         )
 
         symbol_table, error = tests.common.translate_source_to_intermediate(
             source=source
         )
-        assert error is None, f"{error=}"
+        assert error is None, tests.common.most_underlying_messages(error)
 
         assert symbol_table is not None
 
@@ -102,9 +96,10 @@ class Test_parsing_docstrings(unittest.TestCase):
         assert isinstance(some_class, intermediate.Class)
 
         assert some_class.description is not None
+        assert len(some_class.description.remarks) == 1
 
         symbol_references = list(
-            some_class.description.document.findall(
+            some_class.description.remarks[0].findall(
                 condition=intermediate_doc.SymbolReference
             )
         )
@@ -112,11 +107,56 @@ class Test_parsing_docstrings(unittest.TestCase):
         self.assertEqual(1, len(symbol_references))
         self.assertIsInstance(symbol_references[0].symbol, intermediate.Class)
 
+    def test_constraint_and_constraintref(self) -> None:
+        source = textwrap.dedent(
+            '''\
+            class Some_class:
+                """
+                This is some documentation.
+
+                See :constraintref:`AAS-001`.
+
+                :constraint AAS-001:
+                    some constraint
+                """
+
+            __book_url__ = "dummy"
+            __book_version__ = "dummy"
+            '''
+        )
+
+        symbol_table, error = tests.common.translate_source_to_intermediate(
+            source=source
+        )
+        assert error is None, tests.common.most_underlying_messages(error)
+
+        assert symbol_table is not None
+
+        some_class = symbol_table.must_find(Identifier("Some_class"))
+        assert isinstance(some_class, intermediate.Class)
+
+        assert some_class.description is not None
+        assert len(some_class.description.remarks) == 1
+
+        constraint_references = list(
+            some_class.description.remarks[0].findall(
+                condition=intermediate_doc.ConstraintReference
+            )
+        )
+        self.assertEqual(1, len(constraint_references))
+        self.assertEqual("AAS-001", constraint_references[0].reference)
+
+        self.assertListEqual(
+            ["AAS-001"], list(some_class.description.constraints_by_identifier.keys())
+        )
+
 
 class Test_against_recorded(unittest.TestCase):
-    # Set this variable to True if you want to re-record the test data,
-    # without any checks
-    RERECORD = False
+    RERECORD = os.environ.get("AAS_CORE_CODEGEN_RERECORD", "").lower() in (
+        "1",
+        "true",
+        "on",
+    )
 
     def test_cases(self) -> None:
         this_dir = pathlib.Path(os.path.realpath(__file__)).parent
@@ -190,9 +230,9 @@ class Test_against_recorded(unittest.TestCase):
                 error_str = tests.common.most_underlying_messages(error)
 
                 if Test_against_recorded.RERECORD:
-                    expected_error_pth.write_text(error_str)
+                    expected_error_pth.write_text(error_str, encoding="utf-8")
                 else:
-                    expected_error_str = expected_error_pth.read_text()
+                    expected_error_str = expected_error_pth.read_text(encoding="utf-8")
                     self.assertEqual(expected_error_str, error_str, f"{case_dir=}")
 
             else:
@@ -207,9 +247,20 @@ class Test_against_recorded(unittest.TestCase):
                 symbol_table_str = intermediate.dump(symbol_table)
 
                 if Test_against_recorded.RERECORD:
-                    expected_symbol_table_pth.write_text(symbol_table_str)
+                    expected_symbol_table_pth.write_text(
+                        symbol_table_str, encoding="utf-8"
+                    )
                 else:
-                    expected_symbol_table_str = expected_symbol_table_pth.read_text()
+                    try:
+                        expected_symbol_table_str = expected_symbol_table_pth.read_text(
+                            encoding="utf-8"
+                        )
+                    except Exception as exception:
+                        raise RuntimeError(
+                            f"Failed to read the file representing "
+                            f"the expected symbol table: {expected_symbol_table_pth}"
+                        ) from exception
+
                     self.assertEqual(
                         expected_symbol_table_str,
                         symbol_table_str,
